@@ -13,7 +13,12 @@
 
 /* This include generates two functions that are usually empty. */
 
-void stopfunction( void ) { }
-int endfunction( const struct ssPWD *pwd ) { return PWD_FAIL; }
+void stopfunction (void) { }
+int endfunction (const struct ssPWD *pwd)
+  {
+  /* shut up compiler complaining about pwd being unused */
+  (void) pwd;
+  return PWD_FAIL;
+  }
 
 #endif
